@@ -1,5 +1,5 @@
 from collections import OrderedDict
-from typing import OrderedDict as OrderedDictType, Iterator, Mapping
+from typing import Iterator, Mapping
 from dataclasses import dataclass, field
 from functools import partial
 from pathlib import Path
@@ -28,8 +28,8 @@ class TrainingArguments(Mapping):
     optimizer: Optional[Optimizer] = None
     scheduler: Optional[Any] = None
     logdir: Optional[Union[str, Path]] = None
-    loaders: Optional[OrderedDictType[str, DataLoader]] = None
-    callbacks: Optional[Union[List[Callback], OrderedDictType[str, Callback]]] = None
+    loaders: Optional[Dict[str, DataLoader]] = None
+    callbacks: Optional[Union[List[Callback], Dict[str, Callback]]] = None
     fp16: Union[Dict, bool] = None
     catalyst_args: Dict = field(default_factory=lambda: {})
 
@@ -108,8 +108,8 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         if early_stop:
             self.default_callbacks.append(EarlyStoppingCallback(patience=5))
 
+        (self.project_dir / self.default_logdir).mkdir(exist_ok=True)
         if train_test_val_indices is None:
-            (self.project_dir / self.default_logdir).mkdir(exist_ok=True)
             train_test_val_indices = project_split(project.df, self.project_dir / self.default_logdir)
         else:
             save_split(self.project_dir / self.default_logdir, train_test_val_indices)
@@ -153,18 +153,23 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
                                          ("inference", InferDictCallback())])
         kwargs['callbacks'] = kwargs.get('callbacks', default_callbacks)
         kwargs['model'] = kwargs.get('model', self.model)
+        store = kwargs.pop('store', True)
+        kwargs.pop('loader_names_to_skip_in_interpretation', ())
         del kwargs['datasets']
         super().infer(*args, **kwargs)
-        results = kwargs['callbacks']['inference'].predictions
-        targets = kwargs['callbacks']['inference'].targets
-        interpretation = kwargs['callbacks']['interpretation'].interpretations
+        res = {}
+        if 'inference' in kwargs['callbacks']:
+            res['logits'] = kwargs['callbacks']['inference'].predictions
+            res['targets'] = kwargs['callbacks']['inference'].targets
+        if 'interpretation' in kwargs['callbacks']:
+            res['interpretations'] = kwargs['callbacks']['interpretation'].interpretations
 
-        out_dir = logdir / 'infer'
-        out_dir.mkdir(exist_ok=True)
-        torch.save(results, out_dir / 'logits.pkl')
-        torch.save(targets, out_dir / 'targets.pkl')
-        torch.save(interpretation, out_dir / 'interpretations.pkl')
-        return results, targets, interpretation
+        if store:
+            out_dir = logdir / 'infer'
+            out_dir.mkdir(exist_ok=True)
+            for key in res:
+                torch.save(res[key], out_dir / f'{key}.pkl')
+        return res
 
     def create_interpretation_callback(self, **kwargs) -> InterpretationCallback:
         tensorboard_converters = TensorboardConverters(
@@ -172,7 +177,8 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
             tensorboard_loggers=self.tensor_loggers,
             datasets=kwargs.get('datasets', self.get_default_datasets(**kwargs))
         )
-        interpretation_callback = InterpretationCallback(self.task_flow, tensorboard_converters)
+        loaders_to_skip = kwargs.get('loader_names_to_skip_in_interpretation', ())
+        interpretation_callback = InterpretationCallback(self.task_flow, tensorboard_converters, loaders_to_skip)
         return interpretation_callback
 
     def get_default_loaders(self, shuffle_train=True,
@@ -182,10 +188,9 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         train_dataset = datasets['train']
         val_dataset = datasets['valid']
         test_dataset = datasets['test']
-        train_loader = DataLoader(train_dataset, batch_size=batch_size_per_gpu * torch.cuda.device_count(), shuffle=shuffle_train,
-                                  collate_fn=collator)
-        val_loader = DataLoader(val_dataset, batch_size=batch_size_per_gpu * torch.cuda.device_count(), shuffle=False,
-                                collate_fn=collator)
+        bs = max(batch_size_per_gpu, batch_size_per_gpu * torch.cuda.device_count())
+        train_loader = DataLoader(train_dataset, batch_size=bs, shuffle=shuffle_train, collate_fn=collator)
+        val_loader = DataLoader(val_dataset, batch_size=bs, shuffle=False, collate_fn=collator)
         test_loader = DataLoader(test_dataset, batch_size=batch_size_per_gpu * torch.cuda.device_count(), shuffle=False,
                                  collate_fn=collator)
         loaders = OrderedDict({
@@ -240,22 +245,25 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         self.task_flow.get_decoder().load_tuned(tuned_params)
         return model
 
-    def tune(self) -> Dict:
-        predictions, targets, interpretations = self.load_inference_results()
+    def tune(self, store=True) -> Dict:
+        res = self.load_inference_results()
         decoder = self.task_flow.get_decoder()
-        tuned_params = decoder.tune(predictions['valid'], targets['valid'])
-        out_path = self.project_dir / self.default_logdir / 'tuned_params.pkl'
-        torch.save(tuned_params, out_path)
+        tuned_params = decoder.tune(res['logits']['valid'], res['targets']['valid'])
+        if store:
+            out_path = self.project_dir / self.default_logdir / 'tuned_params.pkl'
+            torch.save(tuned_params, out_path)
         return tuned_params
 
-    def load_inference_results(self) -> Tuple[Dict[str, np.ndarray], Dict[str, np.ndarray], Dict[str, np.ndarray]]:
+    def load_inference_results(self) -> Dict:
         logdir = self.project_dir / self.default_logdir
         out_dir = logdir / 'infer'
         out_dir.mkdir(exist_ok=True)
-        results = torch.load(out_dir / 'logits.pkl')
-        targets = torch.load(out_dir / 'targets.pkl')
-        interpretation = torch.load(out_dir / 'interpretations.pkl')
-        return results, targets, interpretation
+        res = {}
+        for key in ['logits', 'targets', 'interpretations']:
+            file_path = out_dir / f'{key}.pkl'
+            if file_path.exists():
+                res[key] = torch.load(out_dir / f'{key}.pkl')
+        return res
 
     def load_tuned(self) -> Dict:
         tuned_params = torch.load(self.project_dir / self.default_logdir / 'tuned_params.pkl')
@@ -263,10 +271,10 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         return tuned_params
 
     def evaluate(self):
-        predictions, targets, interpretations = self.load_inference_results()
+        res = self.load_inference_results()
         self.load_tuned()
         evaluator = self.task_flow.get_evaluator()
-        df = evaluator(predictions['test'], targets['test'])
+        df = evaluator(res['logits']['test'], res['targets']['test'])
         df.to_csv(self.project_dir / self.default_logdir / 'evaluation.csv', index=False)
         return df
 
